package Bio::Tools::PSort::Module::Bayesian;

use Bio::Tools::PSort::Module::OutputI;
use Bio::Tools::PSort::Report::Result;

use Data::Dumper;

use vars qw(@ISA);
@ISA = qw(Bio::Tools::PSort::Module::OutputI);

use strict;

sub new {
  my ($class, @args) = @_;
  my $self = $class->SUPER::new(@args);

  my ($mfile, $prior) = $self->_rearrange([qw(MODEL PRIOR)], @args);

  $self->throw("Bayesian model file not found: $mfile") if(! -e $mfile);

  # Load the Bayesian model produced by the training program.
  open(MFILE, $mfile) || $self->throw("error: open $mfile: $!");
  my $model = join('', <MFILE>);
  close(MFILE) || $self->throw("error: close $model: $!");

  $self->{model} = eval($model);
  $self->throw("Error loading Bayesian model: $@") if($@);

  # Store any prior probabilities provided to us.  (If none provided,
  # each localization will be assumed to have an equal chance of
  # appearing.)
  if(defined($prior)) {
    $self->throw("Prior probabilties must be a hash reference")
      if(ref($prior) ne "HASH");
    $self->{prior} = $prior;
  } else {
    # If we weren't provided a list of prior probabilities, figure out all the
    # localizations we're going to be making predictions for.
    for my $anal (keys(%{$self->{model}})) {
      for my $loc (keys(%{$self->{model}->{$anal}})) {
	$self->{prior}->{$loc} = 1;
      }
    }
  }

  return $self;
}

sub run {
  my ($self, $seq, %res) = @_;
  my (%preds, @res);

  # Initialize to the prior probabilities of getting each localization.
  %preds = %{$self->{prior}};

  for my $anal (keys(%res)) {
    my (@res) = @{$res{$anal}};

    # Ensure that the results from each analysis are the correct object type.
    for my $r (@res) {
      $self->throw("Not a Bio::Tools::PSort::Report::Result object")
	if((! ref($r)) && (! $r->isa("Bio::Tools::PSort::Report::Result")));
    }

    for my $res (@res) {
      # Get the prediction for the analysis.
      my @locs = $res->localization;

      for my $pred (@locs) {
	next if($pred eq "Unknown");

	# Update the probabilities for each location.
	$preds{$_} *= $self->{model}->{$anal}->{$_}->{$pred}
	  for(keys(%preds));
      }
    }
  }

  # Scale the scores so they all sum to 1.0
  my ($tot, $scale);
  $tot += $_ for(values(%preds));
  if($tot) {
    $scale = 1/$tot;
    $preds{$_} *= $scale for(keys(%preds));
  }

  # Sort the scores so the highest comes first.
  push(@res, new Bio::Tools::PSort::Report::Result(-loc => $_, -score => $preds{$_}))
    for(keys(%preds));

  return sort({$b->score <=> $a->score} @res);
}

1;

