package Bio::Tools::PSort::Module::HMMTOP;

use Bio::Tools::PSort::Module::AnalysisI;
use Bio::Tools::PSort::Report::Result;
use Bio::Tools::PSort::Constants qw(:all);

use Bio::Tools::Run::HMMTOP;

use vars qw(@ISA);
@ISA = qw(Bio::Tools::PSort::Module::AnalysisI);

# The hmmtop processes have a habit of going zombie without this.
$SIG{CHLD} = 'IGNORE';

use strict;

sub new {
  my ($class, @args) = @_;
  my $self = $class->SUPER::new(@args);

  my ($dir, $cutoff) = $self->_rearrange([ qw(HMMTOPDIR CUTOFF) ], @args);

  $self->{hmmtop} = new Bio::Tools::Run::HMMTOP(-hmmtopdir => $dir) ||
    $self->throw("Couldn't create new Bio::Tools::HMMTOP object");

  # Set the number of helices we expect for an InnerMembrane protein.
  defined($cutoff) ? $self->{cutoff} = ($cutoff + 0) : $self->{cutoff} = 3;

  return $self;
}

sub run {
  my ($self, $seq, %prev) = @_;

  # Ensure we received a Bio::Seq object.
  $self->throw("Not a Bio::Seq object")
    if((! ref($seq)) && (! $seq->isa("Bio::Seq")));

  my $rep = $self->{hmmtop}->analyze($seq);
  my $num = $rep->num_helices;

  # Check to see if we have data from the Signal module - if we do, we should
  # reduce the helix count IF a signal peptide was detected and a TMH was found
  # before amino acid 70.
  if(exists($prev{Signal})) {
    my ($res) = @{$prev{Signal}};

    if($res->details =~ /^Signal peptide detected/) {
      for(rep->num_helices) {
	$num-- if($_->end <= 70);
      }
    }
  }

  my $details;
  SWITCH: {
      ($num == 0) && do {
	$details = "No internal helices found";
	last SWITCH;
      };

      ($num == 1) && do {
	$details = "1 internal helix found";
	last SWITCH;
      };

      $details = "$num internal helices found";
    };

  if($num >= $self->{cutoff}) {
    return new Bio::Tools::PSort::Report::Result(-details => [ $details ],
						 -score   => 0,
						 -loc     => 'InnerMembrane');
  } else {
    return new Bio::Tools::PSort::Report::Result(-details => [ $details ],
						 -score   => 0,
						 -loc     => 'Unknown');
  }
}

1;
