package Bio::Tools::PSort::Module::Signal;

use Bio::Tools::PSort::Constants qw(:all);
use Bio::Tools::PSort::Module::AnalysisI;
use Bio::Tools::PSort::Report::Result;

use vars qw(@ISA);
use strict;

@ISA = qw(Bio::Tools::PSort::Module::AnalysisI);

sub new {
  my ($class, @args) = @_;
  my $self = $class->SUPER::new(@args);

  my ($svm, $hmm, $prog) = $self->_rearrange([ qw(SVM HMM PROGRAM) ], @args);

  $self->{svm} = $svm || $self->throw("No SVM model specified.");
  $self->{hmm} = $hmm || $self->throw("No HMM model specified.");
  $self->{program} = $prog || $self->throw("No signal program specified. ");

  return $self;
}

sub run {
  my ($self, $seq, %prev) = @_;
  my ($hassig);

  $self->throw("Not a Bio::Seq object")
    if((! ref($seq)) && (! $seq->isa("Bio::Seq")));

  # Check to see if we can detect a signal peptide with the HMM/SVM module.
  my $sq = $seq->seq;

  # Run the program to check for signal peptides.
  open(SIG, "$self->{program} --svm=$self->{svm} --hmm=$self->{hmm} $sq |");
  my $res = <SIG>;
  chomp($res);
  close(SIG);

  $hassig = 1 if($res);

  # If a signal peptide was detected the sequence is one of the non-cytoplasmic
  # localizations.
  if($hassig) {
    my @locs = qw(InnerMembrane Periplasmic OuterMembrane Extracellular);
    my $detail = "Signal peptide detected";
    return new Bio::Tools::PSort::Report::Result(-loc => \@locs,
						 -details => [ $detail ]);
  } else {
    my $detail = "No signal peptide detected";
    return new Bio::Tools::PSort::Report::Result(-loc => "Unknown",
						 -details => [ $detail ]);
  }
}

1;
