=head1 NAME

Bio::Tools::Run::HMMTOP - Wrapper for the HMMTOP transmembrane helix predictor.

=head1 SYNOPSIS

=head1 DESCRIPTION

=head1 CONSTRUCTOR

=head1 SEE ALSO

=head1 AUTHOR

 Cory Spencer <cspencer@sfu.ca>

=cut

package Bio::Tools::Run::HMMTOP;

use Bio::Tools::Run::HMMTOP::Report;
use Bio::Tools::Run::HMMTOP::Helix;

use Bio::Root::Root;
use Bio::Root::IO;

our @ISA = qw(Bio::Root::Root);
our $VERSION = "0.01";

use strict;

sub new {
  my ($class, @args) = @_;
  my $self = $class->SUPER::new(@args);

  ($self->{dir}) = $self->_rearrange([ qw(HMMTOPDIR) ], @args);

  # Make sure the HMMTOP program files exist.
  $self->throw("HMMTOP directory not found") if(! -e $self->{dir});
  $self->throw("HMMTOP program file not found")
    if (! -e Bio::Root::IO->catfile($self->{dir}, "hmmtop"));
  $self->throw("HMMTOP architecture file not found")
    if (! -e Bio::Root::IO->catfile($self->{dir}, "hmmtop.arch"));
  $self->throw("HMMTOP PSV file not found")
    if (! -e Bio::Root::IO->catfile($self->{dir}, "hmmtop.psv"));

  # Set some of the environment variables that HMMTOP needs to run.
  $ENV{HMMTOP_ARCH} = Bio::Root::IO->catfile($self->{dir}, "hmmtop.arch");
  $ENV{HMMTOP_PSV}  = Bio::Root::IO->catfile($self->{dir}, "hmmtop.psv");

  return $self;
}

sub analyze {
  my ($self, $seq) = @_;

  # Ensure we got a Bio::Seq object.
  $self->throw("Not a Bio::Seq object")
    if((! ref($seq)) || (! $seq->isa('Bio::Seq')));

  # Create an object we'll use to get temporary file names.
  my $io = new Bio::Root::IO();

  # Path to the HMMTOP executable.
  my $prog = Bio::Root::IO->catfile($self->{dir}, "hmmtop");

  # Create an temporary file to store the sequence in.
  my ($fh, $name) = $io->tempfile();
  print($fh ">" . $seq->display_id . "\n" . $seq->seq . "\n");
  close($fh);

  # Run HMMTOP and grab the output.
  my $tseq = $seq->seq;
  open(HMMTOP, "$prog -if=$name -sf=FAS |");
  my $output = <HMMTOP>;
  close(HMMTOP);

  my @helices;
  if($output) {
    $output  =~ s!^.+(IN|OUT)\s+!!;
    my @data = split(/\s+/, $output);

    if(@data) {
      my $num = shift(@data);

      for(1..$num) {
	my ($start, $end) = (shift @data, shift @data);
	my $hlx = new Bio::Tools::Run::HMMTOP::Helix(-start  => $start,
						     -end    => $end);

	push(@helices, $hlx);
      }
    }
  }

  return new Bio::Tools::Run::HMMTOP::Report(-helices => \@helices);
}

1;

