package Bio::Tools::Run::SCLBlast;

use Bio::Tools::Run::SCLBlast::Report;
use Bio::Tools::Run::SCLBlast::Hit;

use Bio::Tools::Run::StandAloneBlast;
use Bio::Root::Root;

use strict;

our @ISA = qw(Bio::Tools::Run::StandAloneBlast);
our $VERSION = '0.01';
our %ENV;

sub new {
  my ($class, %args) = @_;
  my $database;

  # Pull out some of the stuff we need before sending it off to the
  # superclass.
  if(exists($args{-database})) {
    $database = $args{-database};
    delete($args{-database});
  }

  if(exists($args{-blastdir})) {
    $args{blastdir} = $args{-blastdir};
    delete($args{-blastdir});
  }

  my @args = %args;
  my $self = $class->SUPER::new(@args);

  # Set some defaults for our parameters.
  $self->d($database) if($database);
  $self->{database} = $database;

  $self->{parser} = sub {
    my $line = shift;

    # Extract the localization/description strings.
    $line =~ /^.*\[(.*)\]\s+(.*)$/;
    my ($loc, $dsc) = ($1, $2);

    # Check to see if we have multiple localizations.
    $loc = [split(/\s+|\,/, $loc)] if($loc =~ /\,/);

    return ($loc, $dsc);
  };

  return $self;
}

sub parser {
  my ($self, $parser) = @_;

  $self->throw("Not a CODE reference") if(ref($parser) ne "CODE");
  $self->{parser} = $parser;
}

sub blast {
  my ($self, $seq) = @_;
  my ($blast, $brep);

  # Ensure we received a Bio::Seq object.
  $self->throw("Not a Bio::Seq object")
    if((! ref($seq)) || (! $seq->isa('Bio::Seq')));

  # Ensure we have a database to read from.
  $self->throw("No SCLBlast database has be selected")
    if(! $self->{database});

  # Create a new Blast object.
  $blast = new Bio::Tools::Run::StandAloneBlast(database => $self->{database},
						program  => 'blastp',
						_READMETHOD => 'Blast');
  $self->throw("Failed to initialize SCLBlast") if(! $blast);

  # Set the default e value.
  $blast->expect(10e-10);

  # Blast the sequence.
  $brep = $blast->blastall($seq);
  return () if(! $brep);

  my $bres = $brep->next_result();

  # Create a list of the hits on the sequence.
  my @hits;
  while(my $hit = $bres->next_hit()) {
    my @hsps;
    while(my $hsp = $hit->next_hsp) {
      my $qper = $hsp->length('hit')/$hsp->length('query') * 100;

      # Add the HSP to the list if it meets our length criteria.
      push(@hsps, $hsp) if(($qper > 80) && ($qper < 120));
    }

    if(@hsps) {
      my ($loc, $desc) = $self->{parser}->($hit->description);


      my $hit = new Bio::Tools::Run::SCLBlast::Hit(
	          '-localization' => $loc,
		  '-description' => $hit->name . ": " . $desc,
		  '-algorithm' => 'blastp',
		  '-name' => 'sclblast',
		  '-length' => $hit->length,
		  '-hsps' => \@hsps);
      push(@hits, $hit);
    }
  }

  return new Bio::Tools::Run::SCLBlast::Report(-hits => \@hits);
}

1;
