package Bio::Tools::SubLoc;

use Algorithm::SVM::DataSet;
use Algorithm::SVM;

use Bio::Root::Root;
use Carp;

# SubLoc localizations.
my %LOCS  = (Cytoplasmic   => 0, InnerMembrane => 1, Periplasmic   => 2,
	     OuterMembrane => 3, Extracellular => 4, Mitochondrial => 5,
	     Nuclear       => 6, Unknown       => 7);

my %LOCSR = (0 => 'Cytoplasmic',   1 => 'InnerMembrane', 2 => 'Periplasmic',
	     3 => 'OuterMembrane', 4 => 'Extracellular', 5 => 'Mitochondrial',
	     6 => 'Nuclear',       7 => 'Unknown');

use strict;

our @ISA = qw(Bio::Root::Root);
our $VERSION = '0.02';

# List of all the amino acid symbols.
my @AA = qw(V L I M F W Y G A P S T C H R K Q E N D);

sub new {
  my ($class, @args) = @_;
  my $self = $class->SUPER::new(@args);

  # Get the model filename.
  my ($model) = $self->_rearrange(["MODEL"], @args);

  # Load model from a file if specified.
  if($model) {
    $self->{svm} = new Algorithm::SVM(Model => $model);
    $self->throw("SubLoc SVM initialization failed") if(! $self->{svm});
  }

  return $self;
}

sub classify {
  my ($self, $seq) = @_;
  my (%count, $len, $ds, $loc);

  # Ensure we have a model loaded.
  $self->throw("No model is currently loaded") if(! $self->{svm});

  # Make sure that we received a sequence object.
  $self->throw("Not a Bio::Seq object")
    if((! ref($seq)) || (! $seq->isa("Bio::SeqI")));

  # Count the occurences of each amino acid in the sequence.
  $count{uc($_)}++ for(split('', $seq->seq()));

  $len = $seq->length();

  # Create the dataset with all of the elements.
  $ds = new Algorithm::SVM::DataSet(Label => 1);
  $ds->attribute($_, ($count{$AA[$_ - 1]} || 0)/$len) for(1..scalar(@AA));

  # Make the SVM prediction.
  $loc = $self->{svm}->predict($ds);

  # Check if a mapping has been provided for the returned label and return
  # the associated value, else just return the results from the SVM.
  return $LOCSR{$loc};
}

sub train {
  my ($self, $data, $opts) = @_;
  my @tset;

  # Ensure we have at least 2 localizations to sort to.
  $self->throw("Training set must have > 1 localization")
    if(keys(%{$data}) < 2);

  # Create a new SVM.
  $self->{svm} = new Algorithm::SVM(Type => 'C-SVC', Kernel => 'radial');
  $self->throw("Error initializing SVM") if(! $self->{svm});

  foreach my $loc (keys(%{$data})) {
    # Ensure we have a valid localization.
    $self->throw("Unknown localization: $_") if(! exists($data->{$loc}));

    # Ensure we have an array ref of something...
    $self->throw("Training set must be an array of Bio::Seq objects")
      if(ref($data->{$loc}) ne "ARRAY");

    for(@{$data->{$loc}}) {
       my (%count, $len, $ds);

       # Ensure we recieved a set of Bio::Seq objects.
       $self->throw("Not a Bio::Seq object") if(ref($_) ne "Bio::Seq");

       # Count the occurences of each amino acid in the sequence.
       $count{uc($_)}++ for(split('', $_->seq()));
       $len = $_->length();

       if($len) {
	 # Create a new dataset and add it to the list.
	 $ds = new Algorithm::SVM::DataSet(Label => $LOCS{$loc});
	 $ds->attribute($_, ($count{$AA[$_-1]} || 0)/$len) for(1..scalar(@AA));

	 push(@tset, $ds);
       }
     }
  }

  # If the Optimize flag has been set, attempt to find the optimal values
  # for the SVM gamma and C parameters.
  if($opts->{Optimize}) {
    $self->{svm}->train(@tset);

    my ($high_g, $high_c, $high_score);

    $high_score = 0;

    my $start_g = $opts->{GammaStart} || 1;
    my $end_g   = $opts->{GammaEnd}   || 100;
    my $inc_g   = $opts->{GammaInc}   || 1;

    my $start_c = $opts->{CStart} || 1;
    my $end_c   = $opts->{CEnd}   || 50;
    my $inc_c   = $opts->{CInc}   || 1;

    for(my $i = $start_g; $i <= $end_g; $i += $inc_g) {
      for(my $j = $start_c; $j <= $end_c; $j += $inc_c) {
	$self->{svm}->gamma($i);
	$self->{svm}->C($j);

	$self->{svm}->retrain();

	my $score = $self->{svm}->validate(5);
	if($score > $high_score) {
	  $self->debug("Accuracy: $score% (Gamma = $i, C = $j)\n");

	  $high_score = $score;
	  $high_g = $i;
	  $high_c = $j;
	}
      }
    }

    $self->{svm}->gamma($high_g);
    $self->{svm}->C($high_c);
    $self->{svm}->retrain();
  } else {
    # No options set, just train on the dataset.
    return $self->{svm}->train(@tset);
  }
}

sub save {
  my ($self, $filename) = @_;

  return $self->{svm}->save($filename);
}

sub load {
  my ($self, $filename) = @_;

  return $self->{svm}->load($filename);
}

=head1 NAME

Bio::Tools::SubLoc - Perl implementation of the SubLoc protein subcellular
localization method.

=head1 SYNOPSIS

  use Bio::Tools::SubLoc;
  use Bio::SeqIO;

  # Load a previously trained model from a file.
  $subloc = new Bio::Tools::SubLoc(-model  => 'subloc.model');

  # Classify a Bio::Seq object.
  $loc = $subloc->classify($seq);

  # Train SubLoc on a new dataset.
  $accuracy = $subloc->train({ Extracellular => \@extr_seqs,
                               Cytoplasmic   => \@cyto_seqs,
                               Periplasmic   => \@peri_seqs },
                             { Optimize      => 1 });

  # Save the model to a file.
  $subloc->save('subloc.model.new');

  # Load a model from a file.
  $subloc->load('subloc.model.new');

=head1 DESCRIPTION

Bio::Tools::Subloc is an implementation of the SubLoc protein subcellular
localization method, which predicts localizations based on amino acid
composition.  The method was originally outlined in "Support Vector
Machine Approach for Protein Subcellular Localization Prediction" paper by
Sujun Hua and Zhirong Sun.

=head1 CONSTRUCTOR

   $subloc = new Bio::Tools::SubLoc(-model  => 'subloc.model');

The SubLoc constructor accepts the name of an existing model file.

=head1 METHODS

   $loc = $sl->classify($seq);

The classify method accepts a Bio::Seq object as an argument and returns
a string containing the predicted localization of the protein.  The
return value will be one of: Cytoplasmic, InnerMembrane, Periplasmic,
OuterMembrane, Secreted, Mitochondrial, Nuclear or Unknown.

  $accuracy = $subloc->train({ Extracellular => \@extr_seqs,
                               Cytoplasmic   => \@cyto_seqs,
                               Periplasmic   => \@peri_seqs },
                             { Optimize      => 1 });

The train method allows the creation of a new SubLoc model based on a
set of user defined sequences.  The first parameter is a hashref whose
keys correspond to any of the possible predicted localizations.  The
value of each key should be a reference to an array of Bio::Seq objects.

The resulting model is unlikely to provide optimal results without some
degree of fine tuning to the underlying Support Vector Machine.  The
second (optional) parameter to the train method is a hashref which allows
one to pass options directly to the SVM.  The set of valid keys are as
follows:

  Optimize   - If true, the module will attempt to search for the best
               values of the gamma and C parameters passed to the SVM.
               For any new model, this should be run at least once.

               By default, the SubLoc module will try all values for gamma
               between 1 and 100 (increments of 1) and C between 1 and 50
              (increments of one) until the optimal values of each are
               located.

               Note that this **WILL** take quite some time, but only
               needs to be done once per model.  The model should be
               saved with the save method after training so that all
               optimal parameter values will be retained.

  GammaStart - Allows specification of the start value when searching for
               the optimal gamma value.  (Default 1)

  GammaEnd   - Allows specification of the end value when searching for
               the optimal gamma value.  (Default 100)

  GammaInc   - Allows specification of the size of the increment when
               searching for the optimal gamma value.  (Default 1)

  CStart      - Allows specification of the start value when searching for
               the optimal C value.  (Default 1)

  CEnd        - Allows specification of the end value when searching for
                the optimal C value.  (Default 50)

  CInc        - Allows specification of the size of the increment when
                searching for the optimal C value.  (Default 1)

  $subloc->save('subloc.model.new');

Saves the currently loaded model to the specified file.  Returns true on
success, false on failure.

  $subloc->load('subloc.model.new');

Loads an existing model from file.  Returns true on success, false on
failure.


=head1 AUTHOR

Cory Spencer <cspencer@sfu.ca>

=head1 SEE ALSO

Algorithm::SVM and the original SubLoc paper titled "Support Vector Machine
Approach for Protein Subcellular Localization Prediction" by
Sujun Hua and Zhirong Sun

=head1 ACKNOWLEGEMENTS

Chih-Jen Lin, one of the authors of the libsvm package upon which
Algorithm::SVM was based, was invaluable in creating this version of SubLoc
Thanks also to Sujun Hua and Zhirong Sun, the authors of the original SubLoc.
Last but not least, thanks to Fiona Brinkman, Jennifer Gardy and the other
members of the Simon Fraser University Brinkman laboratory.

=cut

1;
